using System;
using System.IO;
using System.CodeDom.Compiler;
using System.Collections.Generic;
using Microsoft.VisualStudio.TextTemplating;
using Microsoft.Win32;
using System.Reflection;


namespace TextTemplateHost
{
    //the text template transformation engine is responsible for running 
    //the transformation process
    //the host is responsible for all input and output, locating files, 
    //and and anything else related to the external environment
    //-------------------------------------------------------------------------
    class CustomCmdLineHost : ITextTemplatingEngineHost
    {
        //the path and file name of the text template that is being processed
        //---------------------------------------------------------------------
        public string inputFile;


        //this will be the extension of the generated text output file
        //the host can provide a default by setting the value of the field here
        //the engine can change this based on the optional output directive
        //if the user specifies it in the text template
        //---------------------------------------------------------------------
        private string fileExtensionValue = ".txt";
        public string FileExtension
        {
            get { return fileExtensionValue; }
        }


        //these are the errors that occur when the engine processes a template
        //the engine passes the errors to the host when it is done processing
        //and the host can decide how to display them, for example the the host 
        //can display the errors in the UI or write them to a file
        //---------------------------------------------------------------------
        private CompilerErrorCollection errorsValue;
        public CompilerErrorCollection Errors
        {
            get { return errorsValue; }
        }


        //the host can provide standard assembly references
        //the engine will use these references when compiling and
        //executing the generated transformation class
        //--------------------------------------------------------------
        public IList<string> StandardAssemblyReferences
        {
            get
            {
                return new string[]
        {
            //if this host searches standard paths and the GAC
            //we can specify the assembly name like this
            //---------------------------------------------------------
            //"System"

            //since this host only resolves assemblies from the 
            //fully qualified path and name of the assembly
            //this is a quick way to get the code to give us the
            //fully qualified path and name of the System assembly
            //---------------------------------------------------------
            typeof(System.Uri).Assembly.Location,
            //typeof(Microsoft.VisualStudio.Modeling.Validation.ValidationContext).Assembly.Location,
            //typeof(Microsoft.VisualStudio.Modeling.Diagrams.AnchorPoint).Assembly.Location,
            //typeof(Microsoft.VisualStudio.Modeling.Utilities.Connector).Assembly.Location,
            //typeof(Microsoft.VisualStudio.TextTemplating.VSHost.ModelingTextTransformation).Assembly.Location
            //typeof(Microsoft.VisualStudio.EnterpriseTools.Validation.ValidationContext).Assembly.Location
        };
            }
        }


        //the host can provide standard imports or using statements
        //the engine will add these statements to the generated 
        //transformation class
        //--------------------------------------------------------------
        public IList<string> StandardImports
        {
            get
            {
                return new string[]
                {
                    "System"
                };
            }
        }


        //the engine calls this method based on the optional include directive
        //if the user has specified it in the text template
        //this method can be called 0, 1, or more times
        //---------------------------------------------------------------------
        public string LoadIncludeText(string requestFileName)
        {
            //if the argument is the fully qualified path of an existing file,
            //then we are done
            //----------------------------------------------------------------
            if (File.Exists(requestFileName))
            {
                return File.ReadAllText(requestFileName);
            }

            //this can be customized to search specific paths for the file
            //this can be customized to accept paths to search as command line
            //arguments
            //----------------------------------------------------------------
            else
            {
                return "";
            }
        }


        //the engine calls this method to resolve assembly references used in
        //the generated transformation class project, and for the optional 
        //assembly directive if the user has specified it in the text template
        //this method can be called 0, 1, or more times
        //---------------------------------------------------------------------
        public string ResolveAssemblyReference(string assemblyReference)
        {
            //if the argument is the fully qualified path of an existing file,
            //then we are done (this does not do any work)
            //----------------------------------------------------------------
            if (File.Exists(assemblyReference))
            {
                return assemblyReference;
            }

            //maybe the assembly is in the same folder as the text template that 
            //called the directive
            //----------------------------------------------------------------
            string candidate = Path.Combine(Path.GetDirectoryName(this.inputFile), assemblyReference);
            if (File.Exists(candidate))
            {
                return candidate;
            }

            //this can be customized to search specific paths for the file,
            //or to search the GAC
            //----------------------------------------------------------------
            candidate = Assembly.Load(new AssemblyName(assemblyReference)).Location;
            if (File.Exists(candidate))
            {
                return candidate;
            }
            //this can be customized to accept paths to search as command line
            //arguments
            //----------------------------------------------------------------

            //if we cannot do better - return the original file name
            return "";
        }


        //the engine calls this method based on the directives the user has 
        //specified it in the text template
        //this method can be called 0, 1, or more times
        //---------------------------------------------------------------------
        public Type ResolveDirectiveProcessor(string processorName)
        {
            //check the processor name, and if it is the name of the processor the 
            //host wants to support, return the type of the processor
            //---------------------------------------------------------------------
            if (string.Compare(processorName, "ApplicationModelDirectiveProcessor", StringComparison.InvariantCultureIgnoreCase) == 0)
            {

                try
                {
                    string keyName = @"SOFTWARE\Microsoft\VisualStudio\8.0Exp\TextTemplating\DirectiveProcessors\ApplicationModelDirectiveProcessor";

                    using (RegistryKey specificKey = Registry.LocalMachine.OpenSubKey(keyName))
                    {
                        if (specificKey != null)
                        {
                            List<string> names = new List<String>(specificKey.GetValueNames());

                            string classValue = specificKey.GetValue("Class") as string;
                            if (!string.IsNullOrEmpty(classValue))
                            {
                                string loadValue = string.Empty;
                                System.Reflection.Assembly processorAssembly = null;

                                if (names.Contains("Assembly"))
                                {
                                    loadValue = specificKey.GetValue("Assembly") as string;
                                    if (!string.IsNullOrEmpty(loadValue))
                                    {
                                        processorAssembly = System.Reflection.Assembly.Load(loadValue);
                                        //because we use Assembly.Load, the assembly must be installed in the GAC
                                    }
                                }
                                if (processorAssembly == null)
                                {
                                    throw new Exception("Directive Processor not found");
                                }

                                Type processorType = processorAssembly.GetType(classValue);
                                if (processorType == null)
                                {
                                    throw new Exception("Directive Processor not found");
                                }
                                return processorType;
                            }
                        }
                    }
                }
                catch (Exception e)
                {
                    //if the directive processor can not be found, throw an error
                    throw new Exception("Directive Processor not found");
                }
            }

            //if the directive processor is not one this host wants to support
            throw new Exception("Directive Processor not supported");

        }


        //a directive processor can call this method if a file name does not 
        //have a path
        //the host can attempt to provide path information by searching 
        //specific paths for the file and returning the file and path if found
        //this method can be called 0, 1, or more times
        //---------------------------------------------------------------------
        public string ResolveFileName(string fileName)
        {
            if (fileName == null)
            {
                throw new ArgumentNullException("the file name cannot be null");
            }

            //if the argument is the fully qualified path of an existing file,
            //then we are done
            //----------------------------------------------------------------
            if (File.Exists(fileName))
            {
                return fileName;
            }

            //maybe the file is in the same folder as the text template that 
            //called the directive
            //----------------------------------------------------------------
            string candidate = Path.Combine(Path.GetDirectoryName(this.inputFile), fileName);
            if (File.Exists(candidate))
            {
                return candidate;
            }

            //look more places
            //----------------------------------------------------------------
            //more code can go here...

            //if we cannot do better - return the original file name
            return fileName;
        }



        //if a call to a directive in a text template does not provide a value
        //for a required parameter, the directive processor can try to get it
        //from the host by calling this method
        //this method can be called 0, 1, or more times
        //---------------------------------------------------------------------
        public string ResolveParameterValue(string directiveId, string processorName, string parameterName)
        {
            if (directiveId == null)
            {
                throw new ArgumentNullException("the directiveId cannot be null");
            }
            if (processorName == null)
            {
                throw new ArgumentNullException("the processorName cannot be null");
            }
            if (parameterName == null)
            {
                throw new ArgumentNullException("the parameterName cannot be null");
            }

            //code to provide "hard-coded" parameter values goes here
            //this code depends on the directive processors this host will interact with

            //if we cannot do better - return the empty string
            return String.Empty;
        }


        //the engine calls this method to change the extension of the 
        //generated text output file based on the optional output directive 
        //if the user specifies it in the text template
        //---------------------------------------------------------------------
        public void SetFileExtension(string extension)
        {
            //the parameter extension has a '.' in front of it already
            //--------------------------------------------------------
            fileExtensionValue = extension;
        }


        //the engine calls this method when it is done processing a text
        //template to pass any errors that occurred to the host
        //the host can decide how to display them
        //---------------------------------------------------------------------
        public void LogErrors(CompilerErrorCollection errors)
        {
            errorsValue = errors;
        }


        //this is the application domain that is used to compile and execute
        //the generated transformation class to create the generated text output
        //----------------------------------------------------------------------
        public AppDomain ProvideTemplatingAppDomain(string content)
        {
            //this host will provide a new application domain each time the 
            //engine processes a text template
            //-------------------------------------------------------------
            return AppDomain.CreateDomain("Generation App Domain");

            //this could be changed to return the current appdomain, but new 
            //assemblies are loaded into this AppDomain on a regular basis
            //if the AppDomain lasts too long, it will grow indefintely 
            //which might be regarded as a leak

            //this could be customized to cache the application domain for 
            //a certain number of text template generations (for example 10)

            //this could be customized based on the contents of the text 
            //template, which are provided as a parameter for that purpose
        }



        #region ITextTemplatingEngineHost Members

        public object GetHostOption(string optionName)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public bool LoadIncludeText(string requestFileName, out string content, out string location)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public string ResolvePath(string path)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void SetOutputEncoding(System.Text.Encoding encoding, bool fromOutputDirective)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public string TemplateFile
        {
            get { throw new Exception("The method or operation is not implemented."); }
        }

        #endregion
    }
    
}
