/***************************************************************************

Copyright (c) Microsoft Corporation. All rights reserved.
This code is licensed under the Visual Studio SDK license terms.
THIS CODE IS PROVIDED *AS IS* WITHOUT WARRANTY OF
ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING ANY
IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR
PURPOSE, MERCHANTABILITY, OR NON-INFRINGEMENT.

***************************************************************************/

using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using Microsoft.VisualStudio.OLE.Interop;
using Microsoft.VisualStudio.Shell.Interop;
using Microsoft.VisualStudio.Shell;
using System.Runtime.InteropServices;
using System.Reflection; // to retrieve version info....
using Microsoft.VisualStudio.ProjectAggregator2.Interop;

using MSBuild = Microsoft.Build.BuildEngine;

namespace Microsoft.VisualStudio.Package 
{
	/// <summary>
	/// This is the base class for the project factory
	/// </summary>
	[CLSCompliant(false)]
	[ComVisible(true)]
	public abstract class ProjectFactory : Microsoft.VisualStudio.Shell.Flavor.FlavoredProjectFactoryBase
	{
		#region fields
		private Microsoft.VisualStudio.Shell.Package package;
		private System.IServiceProvider site;

		#endregion

		#region properties
		protected Microsoft.VisualStudio.Shell.Package Package
		{
			get
			{
				return this.package;
			}
		}

		protected System.IServiceProvider Site
		{
			get
			{
				return this.site;
			}
		}

	
		#endregion

		#region ctor
		protected ProjectFactory(Microsoft.VisualStudio.Shell.Package package) 
		{
			this.package = package;
			this.site = package;
		}
		#endregion

		#region abstract methods
		protected abstract ProjectNode CreateProject();
		#endregion

		#region overriden methods
		/// <summary>
		/// Rather than directly creating the project, ask VS to initate the process of
		/// creating an aggregated project in case we are flavored. We will be called
		/// on the IVsAggregatableProjectFactory to do the real project creation.
		/// </summary>
		/// <param name="fileName">Project file</param>
		/// <param name="location">Path of the project</param>
		/// <param name="name">Project Name</param>
		/// <param name="flags">Creation flags</param>
		/// <param name="projectGuid">Guid of the project</param>
		/// <param name="project">Project that end up being created by this method</param>
		/// <param name="canceled">Was the project creation canceled</param>
		protected override void CreateProject(string fileName, string location, string name, uint flags, ref Guid projectGuid, out IntPtr project, out int canceled)
		{
			project = IntPtr.Zero;
			canceled = 0;

			// Get the list of GUIDs from the project/template
			string guidsList = this.ProjectTypeGuids(fileName);

			// Launch the aggregate creation process (we should be called back on our IVsAggregatableProjectFactoryCorrected implementation)
			IVsCreateAggregateProject aggregateProjectFactory = (IVsCreateAggregateProject)this.Site.GetService(typeof(SVsCreateAggregateProject));
			int hr = aggregateProjectFactory.CreateAggregateProject(guidsList, fileName, location, name, flags, ref projectGuid, out project);
			if (hr == VSConstants.E_ABORT)
				canceled = 1;
			ErrorHandler.ThrowOnFailure(hr);

			// This needs to be done after the aggregation is completed (to avoid creating a non-aggregated CCW) and as a result we have to go through the interface
			IProjectEventsProvider eventsProvider = (IProjectEventsProvider)Marshal.GetTypedObjectForIUnknown(project, typeof(IProjectEventsProvider));
			eventsProvider.ProjectEventsProvider = this.GetProjectEventsProvider();
		}

		/// <summary>
		/// Instantiate the project class, but do not proceed with the
		/// initialization just yet.
		/// Delegate to CreateProject implemented by the derived class.
		/// </summary>
		protected override object PreCreateForOuter(IntPtr outerProjectIUnknown)
		{
			return this.CreateProject();
		}
		#endregion

		/// <summary>
		/// Retrives the list of project guids from the project file.
		/// If you don't want your project to be flavorable, override
		/// to only return your project factory Guid:
		///      return this.GetType().GUID.ToString("B");
		/// </summary>
		/// <param name="file">Project file to look into to find the Guid list</param>
		/// <returns>List of semi-colon separated GUIDs</returns>
		protected override string ProjectTypeGuids(string file)
		{
			// Load the project so we can extract the list of GUIDs
			MSBuild.Engine buildEngine = MSBuild.Engine.GlobalEngine;
			buildEngine.BinPath = Utilities.GetMsBuildPath(this.Site);
			MSBuild.Project buildProject = buildEngine.CreateNewProject();
			buildProject.Load(file);

			// Retrieve the list of GUIDs, if it is not specify, make it our GUID
			string guids = buildProject.GetEvaluatedProperty(ProjectFileConstants.ProjectTypeGuids);
			if (String.IsNullOrEmpty(guids))
				guids = this.GetType().GUID.ToString("B");

			buildProject.ParentEngine.UnloadProject(buildProject);

			return guids;
		}

		#region helpers
		private IProjectEvents GetProjectEventsProvider()
		{
			ProjectPackage projectPackage = this.package as ProjectPackage;
			Debug.Assert(projectPackage != null, "Package not inherited from framework");
			if (projectPackage != null)
			{
				foreach (SolutionListener listener in projectPackage.SolutionListeners)
				{
					IProjectEvents projectEvents = listener as IProjectEvents;
					if (projectEvents != null)
					{
						return projectEvents;
					}
				}
			}

			return null;
		}
		#endregion

	}
} // namespace end
