using System;
using System.Runtime.InteropServices;
using System.Collections;
using System.IO;
using System.Windows.Forms;
using System.Diagnostics;
using System.Globalization;
using System.Reflection;
using System.Text;
using System.Threading;
using Microsoft.VisualStudio.Shell;
using Microsoft.VisualStudio.Shell.Interop;
using Microsoft.VisualStudio.OLE.Interop;
//using MSBuild = Microsoft.Build.BuildEngine;
//using Microsoft.Build.Utilities;
using Microsoft.VisualStudio.Package;
using Microsoft.VisualStudio;

namespace Eastbanc.VSIP.Sdm.VSProject
{
    [CLSCompliant(false)]
    [ComVisible(true)]
    public class SdmDocumentReferenceNode : ReferenceNode
	{
        private String  sdmDocumentName;
        private Version sdmDocumentVersion;
        private String sdmDocumentPath;

        protected VSLangProj.Reference _referenceObject;

        #region ctors
		/// <summary>
		/// Constructor for the ReferenceNode
		/// </summary>
        public SdmDocumentReferenceNode(ProjectNode root, ProjectElement e)
            : base(root, e)
        {
			string include = this.ItemNode.GetMetadata(ProjectFileConstants.Include);
            
			CreateFromSdmDocumentName(include);
        }

        public SdmDocumentReferenceNode(ProjectNode root, String filePath)
            : base(root)
        {
            Debug.Assert(filePath != null, "SdmDocument Path is null");
            Debug.Assert(filePath != String.Empty, "SdmDocument Path is empty");

            foreach (String path in filePath.Split(new char[] { '|' }))
            {
                Debug.Assert(File.Exists(path), "SDM Document File does not eaxist.");

                CreateFromSdmDocumentName(Path.GetFileName(path));
            }

        }

        private void CreateFromSdmDocumentName(string include)
        {
            sdmDocumentName = include;
            sdmDocumentVersion = new Version(1, 0);
            sdmDocumentPath = include;
        }

        #endregion

        public override string Caption
        {
            get
            {
                return sdmDocumentName;
            }
        }

        public override string Url
        {
            get
            {
                return sdmDocumentPath;
            }
        }
                
        internal override object Object
        {
            get
            {
                if (_referenceObject == null)
                    _referenceObject = new OASdmDocumentReference(this);

                return _referenceObject;
            }
        }


        /// <summary>
        /// Checks if an assembly is already added. The method parses all references and compares the full assemblynames, or the location of the assemblies to decide whether two assemblies are the same.
        /// </summary>
        /// <returns>true if the assembly has already been added.</returns>
        protected override bool IsAlreadyAdded()
        {
            ReferenceContainerNode referencesFolder = this.ProjectMgr.FindChild(ReferenceContainerNode.ReferencesNodeVirtualName) as ReferenceContainerNode;
            Debug.Assert(referencesFolder != null, "Could not find the References node");
            bool shouldCheckPath = !string.IsNullOrEmpty(this.Url);

            for (HierarchyNode n = referencesFolder.FirstChild; n != null; n = n.NextSibling)
            {
                SdmDocumentReferenceNode sdmDocumentRefererenceNode = n as SdmDocumentReferenceNode;
                if (null != sdmDocumentRefererenceNode)
                {
                    // We will check if the full assemblynames are the same or if the Url of the assemblies is the same.
                    if (String.Compare(sdmDocumentRefererenceNode.sdmDocumentName, this.sdmDocumentName, StringComparison.OrdinalIgnoreCase) == 0 ||
                        (shouldCheckPath && NativeMethods.IsSamePath(sdmDocumentRefererenceNode.Url, this.Url)))
                    {
                        return true;
                    }
                }
            }

            return false;
        }
       
        protected override void ResolveReference()
        {
            
        }

        protected override void BindReferenceData()
        {
            Debug.Assert(this.sdmDocumentName != null, "The AssemblyName field has not been initialized");

            // If the item has not been set correctly like in case of a new reference added it now.
            // The constructor for the AssemblyReference node will create a default project item. In that case the Item is null.
            // We need to specify here the correct project element. 
            if (this.ItemNode == null || this.ItemNode.Item == null)
            {
                //Debug.Assert(false, "Item is not set correctly before BindReference");
                this.ItemNode = new ProjectElement(this.ProjectMgr, this.sdmDocumentName, SdmProjectFileConstants.SdmDocumentReference);
            }

            // Set the basic information we know about
            this.ItemNode.SetMetadata(ProjectFileConstants.Name, this.sdmDocumentName);

            this.SetReferenceProperties();
        }

        /// <summary>
        /// Determines if this is node a valid node for painting the default reference icon.
        /// </summary>
        /// <returns></returns>
        protected override bool CanShowDefaultIcon()
        {
            if (String.IsNullOrEmpty(this.sdmDocumentPath) || !File.Exists(this.sdmDocumentPath))
            {
                return true;
            }
            return true;
        }

        /// <summary>
        /// This function ensures that some properies of the reference are set.
        /// </summary>
        private void SetReferenceProperties()
        {
            // Check if we have to resolve again the path to the assembly.
            if (string.IsNullOrEmpty(this.sdmDocumentPath))
            {
                ResolveReference();
            }
        }
    }
}
